+++
title = "Using Singularity"
description = "How to use the Singularity containerization software on HCC resources."
+++

[Singularity](https://www.sylabs.io/singularity/)
is a containerization solution designed for high-performance computing
cluster environments.  It allows a user on an HPC resource to run an
application using a different operating system than the one provided by
the cluster.  For example, the application may require Ubuntu but the
cluster OS is CentOS.  Conceptually, it is similar to other container
software such as Docker, but is designed with several important
differences that make it more suited for HPC environments.  

- Encapsulation of the environment
- Containers are image based
- No user contextual changes or root escalation allowed
- No root owned daemon processes

To use Singularity on HCC machines, first load the `singularity `module.
Singularity provides a few different ways to access the container.
Most common is to use the `exec` command to run a specific command
within the container; alternatively, the `shell` command is used to
launch a bash shell and work interactively.  Both commands take the
source of the image to run as the first argument.  The `exec` command
takes an additional argument for the command within the container to
run.  Singularity can run images from a variety of sources, including
both a flat image file or a Docker image from Docker Hub.  For
convenience, HCC provides a set of images on
[Docker Hub](https://hub.docker.com/u/unlhcc/)
known to work on HCC resources.  Finally, pass any arguments for the
program itself in the same manner as you would if running it directly.
 For example, the Spades Assembler software is run using the Docker
image `unlhcc/spades` and via the command `spades.py`.
To run the software using Singularity, the commands are:

{{% panel theme="info" header="Run Spades using Singularity" %}}
{{< highlight bash >}}
module load singularity
singularity exec docker://unlhcc/spades spades.py <spades arguments>
{{< /highlight >}}
{{% /panel %}}

Using Singularity in a SLURM job is the same as any other software.

{{% panel theme="info" header="Example Singularity SLURM script" %}}
{{< highlight bash >}}
#!/bin/sh
#SBATCH --time=03:15:00          # Run time in hh:mm:ss
#SBATCH --mem-per-cpu=4096       # Maximum memory required per CPU (in megabytes)
#SBATCH --job-name=singularity-test
#SBATCH --error=/work/[groupname]/[username]/job.%J.err
#SBATCH --output=/work/[groupname]/[username]/job.%J.out

module load singularity
singularity exec docker://unlhcc/spades spades.py <spades arguments>
{{< /highlight >}}
{{% /panel %}}

### Available Images

The following table lists the currently available images and the command
to run the software.

{{% notice note %}}
If you would like to request an image to be added, please fill out the
HCC [Software Request Form](http://hcc.unl.edu/software-installation-request)
and indicate you would like to use Singularity.
{{% /notice %}}

{{< readfile file="static/markdown/singularity-images.md" markdown="true" >}}

### What if I need other Python packages not in the image?
{{% notice info %}}
An alternative to the steps below is to create your own custom image as
[described farther down](#what-if-i-want-to-build-a-custom-image-to-use-on-the-hcc-clusters).
Start with an HCC-provided image as the base for your Dockerfile (i.e. `FROM unlhcc/spades`)
and add any additional packages you desire.
{{% /notice %}}

Unfortunately it's not possible to create one image that has every
available Python package installed for logistical reasons.  Images are
created with a small set of the most commonly-used scientific packages,
but you may need others.  If so, you can install them in a location in
your `$WORK` directory and set the `PYTHONPATH` variable to that
location in your submit script.  The extra packages will then be "seen"
by the Python interpreter within the image.  To ensure the packages will
work, the install must be done from within the container via
the `singularity shell` command.  For example, suppose you are using
the `tensorflow-gpu` image and need the packages `nibabel` and `tables`.
 First, run an interactive SLURM job to get a shell on a worker node.

{{% panel theme="info" header="Run an interactive SLURM job" %}}
{{< highlight bash >}}
srun --pty --mem=4gb --qos=short $SHELL
{{< /highlight >}}
{{% /panel %}}

After the job starts, the prompt will change to indicate you're on a
worker node.  Next, start an interactive session in the container.

{{% panel theme="info" header="Start a shell in the container" %}}
{{< highlight bash >}}
module load singularity
singularity shell docker://unlhcc/tensorflow-gpu
{{< /highlight >}}
{{% /panel %}}

This may take a few minutes to start.  Again, the prompt will change and
begin with `Singularity` to indicate you're within the container.

Next, install the needed packages via `pip` to a location somewhere in
your `work` directory.  For example, `$WORK/tf-gpu-pkgs`.  (If you are
using Python 3, use `pip3` instead of `pip`).

{{% panel theme="info" header="Install needed Python packages with pip" %}}
{{< highlight bash >}}
export LC_ALL=C
pip install --system --target=$WORK/tf-gpu-pkgs --install-option="--install-scripts=$WORK/tf-gpu-pkgs/bin" nibabel tables
{{< /highlight >}}
{{% /panel %}}

You should see some progress indicators, and a
"`Successfully installed..."` message at the end.  Exit both the
container and the interactive SLURM job by typing `exit` twice.  The
above steps only need to be done once per each image you need additional
packages for.   Be sure to use a separate location for each image's
extra packages.

To make the packages visible within the container, you'll need to add a
line to the submit script used for your Singularity job.  Before the
lines to load the `singularity `module and run the script, add a line
setting the `PYTHONPATH` variable to the `$WORK/tf-gpu-pkgs` directory.
For example,

{{% panel theme="info" header="Example SLURM script" %}}
{{< highlight bash >}}
#!/bin/sh
#SBATCH --time=03:15:00          # Run time in hh:mm:ss
#SBATCH --mem-per-cpu=4096       # Maximum memory required per CPU (in megabytes)
#SBATCH --job-name=singularity-test
#SBATCH --partition=gpu
#SBATCH --gres=gpu
#SBATCH --error=/work/[groupname]/[username]/job.%J.err
#SBATCH --output=/work/[groupname]/[username]/job.%J.out
 
export PYTHONPATH=$WORK/tf-gpu-pkgs
module load singularity
singularity exec docker://unlhcc/tensorflow-gpu python /path/to/my_tf_code.py
{{< /highlight >}}
{{% /panel %}}

The additional packages should then be available for use by your Python
code running within the container.

### What if I need a specific software version of the Singularity image?

You can see all the available versions of the software built with
Singularity in the table above. If you don't specify a specific sofware
version, Singulariy will use the latest one. If you want to use a
specific version instead, you can append the version number from the
table to the image. For example, if you want to use the Singularity
image for Spades version 3.11.0, run:

{{< highlight bash >}}
singularity exec docker://unlhcc/spades:3.11.0 spades.py
{{< /highlight >}}

### What if I want to build a custom image to use on the HCC clusters?

You can create custom Docker image and use it with Singularity on our clusters.
Singularity can run images directly from Docker Hub, so you don't need to upload anything to HCC.
For this purpose, you just need to have a Docker Hub account and upload
your image there. Then, if you want to run the command "*mycommand*"
from the image "*myimage*", type:

{{< highlight bash >}}
module load singularity
singularity exec docker://myaccount/myimage mycommand
{{< /highlight >}}

where "*myaccount*" is your Docker Hub account.

In case you see the error `ERROR MANIFEST_INVALID: manifest invalid`
when running the command above, try:

{{< highlight bash >}}
module load singularity
unset REGISTRY
singularity exec docker://myaccount/myimage mycommand
{{< /highlight >}}

{{% notice info %}}
If you get the error `FATAL: kernel too old` when using your Singularity image on the HCC clusters, that means the *glibc* version in your image is too new for the kernel on the cluster. One way to solve this is to use lower version of your base image (for example, if you have used Ubuntu:18.04 please use Ubuntu:16.04 instead).
{{% /notice %}}


All the Dockerfiles of the images we host on HCC are
[publicly available here](https://github.com/unlhcc/singularity-dockerfiles)
You can use them as an example when creating your own image. The only thing you need to note
when creating custom Docker images you want to use on HCC is to add the line:
{{< highlight batch >}}
RUN mkdir -p /work
{{< /highlight >}}
at the end of your Dockerfile. This creates a `/work` directory inside your image so
your `$WORK` directory on Crane/Tusker is available.
