+++
title = "Running BWA Commands"
description =  "How to run BWA commands on HCC resources"
weight = "10"
+++

## BWA Index

The first step of using BWA is to make an index of the reference genome in fasta format. The basic usage of the **bwa index** is:
{{< highlight bash >}}
$ bwa index [-a bwtsw|is] input_reference.fasta index_prefix
{{< /highlight >}}
where **input_reference.fasta** is an input file of the reference genome in fasta format, and **index_prefix** is the prefix of the generated index files. The option **-a** is required and can have two values: **bwtsw** (does not work for short genomes) and **is** (does not work for long genomes). Therefore, this value is chosen according to the length of the genome.


## BWA Mem

The **bwa mem** algorithm is one of the three algorithms provided by BWA. It performs local alignment and produces alignments for different part of the query sequence. The basic usage of **bwa mem** is:
{{< highlight bash >}}
$ bwa mem index_prefix [input_reads.fastq|input_reads_pair_1.fastq input_reads_pair_2.fastq] [options]
{{< /highlight >}}
where **index_prefix** is the index for the reference genome generated from **bwa index**, and **input_reads.fastq**, **input_reads_pair_1.fastq**, **input_reads_pair_2.fastq** are the input files of sequencing data that can be single-end or paired-end respectively. Additional **options** for **bwa mem** can be found in the BWA manual.


Simple SLURM script for running **bwa mem** on Tusker with paired-end fastq input data, `index_prefix` as reference genome index, SAM output file and `8 CPUs` is shown below:
{{% panel header="`bwa_mem.submit`"%}}
{{< highlight bash >}}
#!/bin/sh
#SBATCH --job-name=Bwa_Mem
#SBATCH --nodes=1
#SBATCH --ntasks-per-node=8
#SBATCH --time=168:00:00
#SBATCH --mem=10gb
#SBATCH --output=BwaMem.%J.out
#SBATCH --error=BwaMem.%J.err

module load bwa/0.7

bwa mem index_prefix input_reads_pair_1.fastq input_reads_pair_2.fastq -t $SLURM_NTASKS_PER_NODE > bwa_mem_alignments.sam
{{< /highlight >}}
{{% /panel %}}


## BWA Bwasw

The **bwa bwasw** algorithm is another algorithm provided by BWA. For input files with single-end reads it aligns the query sequences. For input files with paired-ends reads it performs paired-end alignment that only works for Illumina reads.

An example of **bwa bwasw** for single-end input file `input-reads.fasta` in fasta format and output file `bwa_bwasw_alignments.sam` where the alignments are stored, is shown below:
{{< highlight bash >}}
$ bwa bwasw index_prefix input_reads.fasta -t $SLURM_NTASKS_PER_NODE > bwa_bwasw_alignments.sam
{{< /highlight >}}


## BWA Aln

The third BWA algorithm, **bwa aln**, aligns the input file of sequence data to the reference genome. In addition, there is an example of running **bwa aln** with single-end `input_reads.fasta` input file and `8 CPUs`:
{{< highlight bash >}}
$ bwa aln index_prefix input_reads.fasta -0 -t $SLURM_NTASKS_PER_NODE > bwa_aln_alignments.sai
{{< /highlight >}}


## BWA Samse and BWA Sampe

The command **bwa samse** uses the `bwa_aln_alignments.sai` output from **bwa aln** in order to generate SAM file from the alignments for single-end reads.

{{% panel header="`General BWA Samse Usage`"%}}
{{< highlight bash >}}
$ bwa samse -f bwa_aln_alignments.sam index_prefix bwa_aln_alignments.sai input_reads.fasta   output31.preArc
{{< /highlight >}}
{{% /panel %}}

The command **bwa sampe** uses the `bwa_aln_alignments.sai` output form **bwa aln** in order to generate SAM file from the alignments for paired-end reads.

{{% panel header="`General BWA Sampe Usage`"%}}
{{< highlight bash >}}
$ bwa samse -f bwa_aln_alignments.sam index_prefix bwa_aln_alignments_pair_1.sai bwa_aln_alignments_pair_2.sai input_reads_pair_1.fasta input_reads_pair_2.fasta
{{< /highlight >}}
{{% /panel %}}


## BWA Fastmap

The command **bwa fastmap** identifies and outputs super-maximal exact matches (SMEMs). The basic usage of **bwa fastmap** is:
{{< highlight bash >}}
$ bwa fastmap index_prefix input_reads.fasta > bwa_fastmap.matches
{{< /highlight >}}


## BWA Pemerge

The command **bwa pemerge** merges overlapping paired ends and can print either only the merged reads or the unmerged ones. An example of **bwa pemerge** of `input_reads_pair_1.fastq` and `input_reads_pair_2.fastq` with `8 CPUs` and output file `output_reads_merged.fastq` that contains only the merged reads is shown below:
{{< highlight bash >}}
$ bwa pemerge -m input_reads_pair_1.fastq input_reads_pair_2.fastq -t $SLURM_NTASKS_PER_NODE > output_reads_merged.fastq
{{< /highlight >}}


## BWA Fa2pac

The command **bwa fa2pac** converts fasta to pac files. The general usage of **bwa fa2pac** is:
{{< highlight bash >}}
$ bwa fa2pac input_reads.fasta pac_prefix
{{< /highlight >}}


## BWA Pac2bwt and BWA Pac2bwtgen

The commands **bwa pac2bwt** and **bwa pac2bwtgen** convert pac to bwt files.

{{% panel header="`General BWA Pac2bwt Usage`"%}}
{{< highlight bash >}}
$ bwa pac2bwt input_reads.pac output_reads.bwt
{{< /highlight >}}
{{% /panel %}}

{{% panel header="`General BWA Pac2bwtgen Usage`"%}}
{{< highlight bash >}}
$ bwa pac2bwtgen input_reads.pac output_reads.bwt
{{< /highlight >}}
{{% /panel %}}


## BWA Bwtupdate

The command **bwa bwtupdate** updates bwt files to the new format. The general usage of **bwa bwtupdate** is:
{{< highlight bash >}}
$ bwa bwtupdate input_reads.bwt
{{< /highlight >}}


## BWA Bwt2sa

The command **bwa bwt2sa** generates sa files from bwt and Occ files. The basic usage of **bwa bwt2sa** is:
{{< highlight bash >}}
$ bwa bwt2sa input_reads.bwt output_reads.sa
{{< /highlight >}}


### Useful Information

In order to test the scalability of BWA (bwa/0.7) on Crane, we used two paired-end input fastq files, `large_1.fastq` and `large_2.fastq`, and one single-end input fasta file, `large.fasta`. Some statistics about the input files and the time and memory resources used by **bwa mem** are shown on the table below:
{{< readfile file="/static/html/bwa.html" >}}
